@@ -1,5 +1,5 @@
 ;;;; File: g-wrap.scm
-;;;; Copyright (C) 2004-2005 Andreas Rottmann
+;;;; Copyright (C) 2004-2006 Andreas Rottmann
 ;;;;
 ;;;; based upon G-Wrap 1.3.4,
 ;;;;   Copyright (C) 1996, 1997,1998 Christopher Lee
@@ -30,6 +30,7 @@
 
 (define-module (g-wrap)
   #:use-module (ice-9 optargs)
+  #:use-module (ice-9 pretty-print)
   #:use-module (oop goops)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-11)
@@ -51,28 +52,23 @@
 
    <gw-function>
    c-name
-   argument-count input-argument-count optional-argument-count
+   argument-count input-argument-count output-argument-count optional-argument-count
    arguments argument-types argument-typespecs
+   input-arguments output-arguments optional-arguments
    return-type return-typespec
    generic-name 
    
    <gw-type>
-   class-name
    needs-result-var?
+   default-c-value-for-type
    wrap-value-cg unwrap-value-cg destroy-value-cg
    pre-call-arg-cg pre-call-result-cg call-arg-cg post-call-result-cg
    post-call-arg-cg call-cg set-value-cg
-   
-   global-declarations-cg global-definitions-cg
-   declarations-cg initializations-cg init-finalizations-cg
-   
-   client-global-declarations-cg client-global-definitions-cg
-   client-initializations-cg
-   
+
    gen-c-tmp-name
-   
+
    <gw-typespec>
-   type options c-type-name all-types add-option!
+   type options c-type-name all-types add-option! typespec-options
    make-typespec check-typespec-options parse-typespec-option!
    
    <gw-value>
@@ -84,11 +80,11 @@
    <gw-param>
    number output-param?
    
-   <gw-code>
-   render no-op?
+   <gw-code> <gw-scm-code>
+   render no-op? make-scm-code
    has-error-form? expand-special-forms
-   
-   <gw-wrapset>
+
+   <gw-wrapset-class> <gw-wrapset>
    name language wrapsets-depended-on
    fold-types for-each-type lookup-type fold-functions
    consider-types?
@@ -101,7 +97,7 @@
    
    wrap-type! wrap-function! wrap-constant!
 
-   generate-wrapset
+   get-wrapset generate-wrapset compute-client-types
    ))
 
 ;;; Conditions
@@ -164,14 +160,10 @@
 (define-method (all-types-referenced (sel <gw-item>))
   '())
 
-;; Upgrade the GOOPS class-name procedure
-;; barfs with GUILE CVS: class-name not defined -- jcn
-(if (defined? 'class-name)
-    (define class-name (ensure-accessor class-name))
-    (define class-name (ensure-accessor (module-ref
-					 (resolve-module '(oop goops))
-					 'class-name))))
+;; Upgrade the GOOPS class-name procedure.
+(set! class-name (ensure-accessor class-name))
 
+
 ;;;
 ;;; Types
 ;;;
@@ -196,6 +188,14 @@
     (display (name type) port)
     (display #\> port)))
 
+;; Return a default C value for type (a string), or `#f' so that C variables
+;; (in particular those for the result and `out' arguments) are not left
+;; uninitialized.  In the general case, we aren't able to provide such a
+;; value.
+(define-generic default-c-value-for-type)
+(define-method (default-c-value-for-type (type <gw-type>))
+  #f)
+
 (define-method (gen-c-tmp-name (type <gw-type>) (suffix <string>))
   (gen-c-tmp (string-append (any-str->c-sym-str
                              (symbol->string
@@ -231,7 +231,7 @@
 ;;;
 
 (define-class <gw-value> ()
-  (typespec #:getter typespec #:init-keyword #:typespec)
+  (typespec #:getter typespec #:init-keyword #:typespec #:init-value #f)
   (var #:getter var #:init-keyword #:var)
   (wrapped-var #:getter wrapped-var #:init-keyword #:wrapped-var))
 
@@ -244,19 +244,48 @@
                      ((and (list? code2-opt) (= (length code2-opt) 1))
                       (car code2-opt))
                      (else (error "bogus parameters")))))
-    (if (memq option (options (typespec value)))
-        code1
-        (if code2 code2 '()))))
+    (if (typespec value)
+	(if (memq option (options (typespec value)))
+	    code1
+	    (if code2 code2 '()))
+	(if code2 code2 '()))))
 
 (define-generic wrap-value-cg)
 (define-generic unwrap-value-cg)
 (define-generic destroy-value-cg)
 
-(define-method (destroy-value-cg (type <gw-type>) (value <gw-value>) err)
+(define-method (wrap-value-cg (type <gw-type>) (value <gw-value>) error-var
+			      (inlined? <boolean>))
+  ;; This method allows to support `wrap-value-cg' methods that do _not_
+  ;; support the INLINED? argument.
+  (wrap-value-cg type value error-var))
+
+(define-method (wrap-value-cg (type <gw-type>) (value <gw-value>) error-var)
+  (wrap-value-cg type value error-var #f))
+
+(define-method (unwrap-value-cg (type <gw-type>) (value <gw-value>) error-var
+				(inlined? <boolean>))
+  ;; Likewise.
+  (unwrap-value-cg type value error-var))
+
+(define-method (unwrap-value-cg (type <gw-type>) (value <gw-value>) error-var)
+  (unwrap-value-cg type value error-var #f))
+
+(define-method (destroy-value-cg (type <gw-type>) (value <gw-value>) error-var
+				 (inlined? <boolean>))
+  ;; Likewise.
+  (destroy-value-cg type value error-var))
+
+(define-method (destroy-value-cg (type <gw-type>) (value <gw-value>) error-var)
+  (destroy-value-cg type value error-var #f))
+
+(define-method (destroy-value-cg (type <gw-type>) (value <gw-value>) err
+				 (inlined? <boolean>))
   '())
 
 (define-method (pre-call-arg-cg (type <gw-type>) (param <gw-value>) err)
-  (unwrap-value-cg type param err))
+  ;; Call `unwrap-value-cg' in such a way that inlined code is generated.
+  (unwrap-value-cg type param err #t))
 
 ;; What was that for?
 ;;    "if (" `(gw:error? ,status-var type) ")"
@@ -279,17 +308,21 @@
 (define-method (post-call-result-cg (type <gw-type>)
                                     (result <gw-value>)
                                     status-var)
+  ;; Call `wrap-value-cg' and `destroy-value-cg' in such a way that inlined
+  ;; code is generated.
   (list
-   (wrap-value-cg type result status-var)
-   (destroy-value-cg type result status-var)))
+   (wrap-value-cg type result status-var #t)
+   (destroy-value-cg type result status-var #t)))
 
 
 (define-method (post-call-arg-cg (type <gw-type>) (param <gw-value>) err)
+  ;; Call `wrap-value-cg' and `destroy-value-cg' in such a way that inlined
+  ;; code is generated.
   (list
    (if (memq 'out (options (typespec param)))
-       (wrap-value-cg type param err)
+       (wrap-value-cg type param err #t)
        '())
-  (destroy-value-cg type param err)))
+  (destroy-value-cg type param err #t)))
 
 (define-method (set-value-cg (type <gw-type>) (lvalue <gw-value>) rvalue)
   (list (var lvalue) " = " rvalue ";\n"))
@@ -310,6 +343,9 @@
   (type #:init-keyword #:type #:getter type)
   (options #:init-keyword #:options #:getter options #:init-value '()))
 
+(define-method (typespec-options (typespec <gw-typespec>))
+  (options typespec))
+
 (define-method (write (self <gw-typespec>) port)
   (let ((class (class-of self)))
     (display "#<" port)
@@ -381,12 +417,39 @@
 (define-method (argument-count (func <gw-function>))
   (length (slot-ref func 'arguments)))
 
+(define-method (input-arguments (func <gw-function>))
+  (filter (lambda (arg)
+           (and (visible? arg)
+                (not (memq 'out (options (typespec arg))))))
+         (slot-ref func 'arguments)))
+
 (define-method (input-argument-count (func <gw-function>))
   (count (lambda (arg)
            (and (visible? arg)
                 (not (memq 'out (options (typespec arg))))))
          (slot-ref func 'arguments)))
 
+(define-method (output-argument-count (func <gw-function>))
+  (count (lambda (arg)
+           (memq 'out (options (typespec arg))))
+         (slot-ref func 'arguments)))
+
+(define-method (output-arguments (func <gw-function>))
+  (filter (lambda (arg)
+            (memq 'out (options (typespec arg))))
+         (slot-ref func 'arguments)))
+
+(define-method (optional-arguments (func <gw-function>))
+  (let loop ((args (reverse (slot-ref func 'arguments))) (opt-args '()))
+    (cond ((or (null? args) (and (visible? (car args)) 
+                                 (not (default-value (car args)))))
+           (reverse opt-args))
+          ((and (visible? (car args))
+                (not (memq 'out (options (typespec (car args))))))
+           (loop (cdr args) (cons (car args) opt-args)))
+          (else
+           (loop (cdr args) opt-args)))))
+
 ;; Returns the number of optional argument (number of consecutive
 ;; arguments with default values at the end of the argument list).
 (define-method (optional-argument-count (func <gw-function>))
@@ -406,6 +469,9 @@
 (define-method (argument-typespecs (func <gw-function>))
   (map typespec (slot-ref func 'arguments)))
 
+(define-method (return-type (func <gw-function>))
+  (type (return-typespec func)))
+
 (define-method (all-types-referenced (func <gw-function>))
   (fold (lambda (typespec rest)
           (append (all-types typespec) rest))
@@ -442,10 +508,20 @@
   (all-types (typespec self)))
 
 ;;;
-;;; Code (currently nested string lists)
+;;; Code 
 ;;;
 (define-class <gw-code> ())
+(define-class <gw-scm-code> (<gw-code>)
+  (expression #:getter expression #:init-keyword #:expression))
 
+(define (make-scm-code expression)
+  (make <gw-scm-code> #:expression expression))
+
+(define-method (render (code <gw-scm-code>) (port <port>))
+  (if (not (null? (expression code)))
+      (pretty-print (expression code) port)))
+
+;; default representation: nested string lists
 (define-method (render (code <list>) (port <port>))
   (flatten-display code port))
 
@@ -710,43 +786,8 @@
 ;;;
 ;;; Generation
 ;;;
-(define (output-initializer-cgs wrapset cgs port)
-  (let* ((error-var (gen-c-tmp "error_var"))
-         (wrapset-name (name wrapset))
-         (wrapset-name-c-sym (any-str->c-sym-str
-                              (symbol->string wrapset-name)))
-         (wrapset-init-func (string-append "gw_initialize_wrapset_"
-                                           wrapset-name-c-sym)))
 
-    (define (output-initializer-cg cg)
-      (let ((code (cg error-var)))
-        (if (not (null? code))
-            (begin
-              (render (expand-special-forms code #f '(type range memory misc))
-                      port)
-              (if (has-error-form? code)
-                  (flatten-display
-                   (list
-                    "if ((" error-var ").status != GW_ERR_NONE)\n"
-                    "  gw_handle_wrapper_error (gw__arena, &" error-var ",\n"
-                    "                            \"" wrapset-init-func "\",\n"
-                    "                            0);\n")
-                   port))))))
-    
-    (flatten-display
-     (list "{\n"
-           "  GWError " error-var ";\n"
-           "   " error-var ".status = GW_ERR_NONE;\n"
-           "   " error-var ".data = SCM_UNSPECIFIED;\n"
-           "   " error-var ".message = NULL;\n"
-           "   (void) " error-var ";\n")
-     port)
-
-    (for-each (lambda (cg) (output-initializer-cg cg)) cgs)
-
-    (display "}\n" port)))
-
-;; Entry main point for wrapset generation
+;; Main entry point for wrapset generation
 (define-method (generate-wrapset (lang <symbol>)
                                  (name <symbol>)
                                  (basename <string>))
@@ -771,186 +812,6 @@
      (all-types-referenced ws))
     (hash-fold (lambda (key val rest) (cons val rest)) '() client-type-hash)))
 
-(define (generate-wrapset-cs wrapset port)
-  (define (dsp-list lst)
-    (for-each (lambda (s) (display s port)) lst))
-
-  (define (render-client cg)
-      (for-each (lambda (ws)
-                  (render (cg ws) port))
-                (wrapsets-depended-on wrapset)))
-  
-  (define (render-client-items cg)
-      (for-each (lambda (ws)
-                  (for-each (lambda (item)
-                              (render (cg ws item) port))
-                            (reverse (slot-ref ws 'client-items))))
-              (wrapsets-depended-on wrapset)))
-  
-  (let ((wrapset-name-c-sym (any-str->c-sym-str
-                             (symbol->string (name wrapset))))
-        (client-types (compute-client-types wrapset))
-        (items (reverse (slot-ref wrapset 'items))))
-
-    (define (render-items cg)
-      (for-each (lambda (item)
-                  (render (cg wrapset item) port))
-                items))
-    
-    (define (render-client-types cg)
-      (for-each (lambda (type)
-                  (render (cg wrapset type) port))
-                client-types))
-    
-    ;;(format #t "client types: ~S\n" client-types)
-    (dsp-list
-     (list
-      "/* Generated by G-Wrap-TNG: an experimental wrapper engine */\n"
-      "\n"))
-    
-    (render-items before-includes-cg)
-    (render-client-items before-includes-cg)
-    
-    (render (global-declarations-cg wrapset) port)
-    (render-items global-declarations-cg)
-
-    (render-client-items global-declarations-cg)
-    (render-client client-global-declarations-cg)
-    (render-client-types client-global-declarations-cg)
-
-    (dsp-list
-     (list
-      "void gw_initialize_wrapset_" wrapset-name-c-sym "(GWLangArena);\n"))
-    
-    (render (global-definitions-cg wrapset) port)
-    (render-items global-definitions-cg)
-
-    (render-client-items global-definitions-cg)
-    (render-client client-global-definitions-cg)
-    (render-client-types client-global-definitions-cg)
-    
-    ;; The initialization function
-    (dsp-list
-     (list
-      "void\n"
-      "gw_initialize_wrapset_" wrapset-name-c-sym "(GWLangArena gw__arena) {\n"
-      "  static int gw_wrapset_initialized = 0;\n"
-      "\n"))
-
-    (render (declarations-cg wrapset) port)
-    (render-items declarations-cg)
-    
-    (dsp-list
-     (list
-      "  if(gw_wrapset_initialized)\n"
-      "   return;\n"
-      ;"   scm_block_gc++;\n"
-      "\n"))
-
-    ;; TODO: deobfuscate
-    (output-initializer-cgs
-     wrapset
-     (append!
-      (list (lambda (error-var)
-              (initializations-cg wrapset error-var)))
-      (map (lambda (item)
-             (lambda (error-var)
-               (client-initializations-cg wrapset item error-var)))
-           client-types)
-      (map (lambda (item)
-             (lambda (error-var)
-               (initializations-cg wrapset item error-var)))
-           (let-values (((types others) (partition! (lambda (item)
-                                                      (is-a? item <gw-type>))
-                                                    items)))
-             (append! types others)))
-      (list (lambda (error-var)
-              (init-finalizations-cg wrapset error-var)))
-      (map (lambda (item)
-             (lambda (error-var)
-               (init-finalizations-cg wrapset item error-var)))
-           items))
-     port)
-
-    
-    (dsp-list
-     (list
-      ;"    scm_block_gc--;\n"
-      "    gw_wrapset_initialized = 1;\n"
-      "}\n"))))
-
-
-(define-method (generate-wrapset (lang <symbol>)
-                                 (wrapset <gw-wrapset>)
-                                 (basename <string>))
-  (let ((wrapset-source-name (string-append basename ".c"))
-        (client-types (compute-client-types wrapset)))
-
-    (call-with-output-file/cleanup
-     wrapset-source-name
-     (lambda (port)
-       (generate-wrapset-cs wrapset port)))))
-
-;;; Default implementations (no-ops)
-
-(define-method (before-includes-cg (wrapset <gw-wrapset>)
-                                   (item <gw-item>))
-  '())
-
-(define-method (global-declarations-cg (wrapset <gw-wrapset>))
-  '())
-
-(define-method (global-declarations-cg (wrapset <gw-wrapset>)
-                                       (item <gw-item>))
-  '())
-
-(define-method (client-global-declarations-cg (wrapset <gw-wrapset>))
-  '())
-
-(define-method (client-global-declarations-cg (wrapset <gw-wrapset>)
-                                              (item <gw-item>))
-  '())
-
-(define-method (global-definitions-cg (wrapset <gw-wrapset>))
-  '())
-
-(define-method (global-definitions-cg (wrapset <gw-wrapset>)
-                                      (item <gw-item>))
-  '())
-
-(define-method (client-global-definitions-cg (wrapset <gw-wrapset>))
-  '())
-
-(define-method (client-global-definitions-cg (wrapset <gw-wrapset>)
-                                             (item <gw-item>))
-  '())
-
-(define-method (declarations-cg (wrapset <gw-wrapset>))
-  '())
-
-(define-method (declarations-cg (wrapset <gw-wrapset>) (item <gw-item>))
-  '())
-
-(define-method (initializations-cg (wrapset <gw-wrapset>) error-var)
-  '())
-
-(define-method (initializations-cg (wrapset <gw-wrapset>)
-                                   (item <gw-item>)
-                                   error-var)
-  '())
-
-(define-method (client-initializations-cg (wrapset <gw-wrapset>)
-                                          (item <gw-item>)
-                                          error-var)
-  '())
-
-(define-method (init-finalizations-cg (wrapset <gw-wrapset>) err)
-  '())
-
-(define-method (init-finalizations-cg (wrapset <gw-wrapset>)
-                                      (item <gw-item>) err)
-  '())
-
 
 ;;;
 ;;; Code expansion
