@@ -125,6 +125,8 @@ unsigned int gw_test_gw_standard_get_uint_max() { return UINT_MAX; }
 long gw_test_gw_standard_get_long_max() { return LONG_MAX; }
 long gw_test_gw_standard_get_long_min() { return LONG_MIN; }
 unsigned long gw_test_gw_standard_get_ulong_max() { return ULONG_MAX; }
+ssize_t gw_test_gw_standard_get_ssize_max() { return SSIZE_MAX; }
+ssize_t gw_test_gw_standard_get_ssize_min() { return (-SSIZE_MAX - 1); }
 
 int
 gw_test_gw_standard_echo_int (int arg)
@@ -150,6 +152,12 @@ gw_test_gw_standard_echo_unsigned_long (unsigned long arg)
   return arg;
 }
 
+ssize_t
+gw_test_gw_standard_echo_ssize (ssize_t arg)
+{
+  return arg;
+}
+
 char *
 gw_test_gw_standard_echo_mchars_caller_owned (char *arg)
 {
@@ -249,4 +257,114 @@ gw_test_child_pass_back_parent_obj(gwTestParentObj* x)
   return x;
 }
 
+
+/* Aggregating object.  */
 
+struct _gwTestAggregatingObj
+{
+  struct _gwTestAggregatingObj *aggregated;
+  size_t ref_count;
+};
+
+gwTestAggregatingObj *
+gw_test_make_simple_aggregating_obj (void)
+{
+  gwTestAggregatingObj *obj;
+
+  obj = malloc (sizeof (*obj));
+  if (!obj)
+    return NULL;
+
+  obj->aggregated = NULL;
+  obj->ref_count = 0;
+
+  return obj;
+}
+
+gwTestAggregatingObj *
+gw_test_make_aggregating_obj (gwTestAggregatingObj *aggregated)
+{
+  gwTestAggregatingObj *obj;
+
+  obj = gw_test_make_simple_aggregating_obj ();
+  obj->aggregated = aggregated;
+
+  /* Increase the reference counter of AGGREGATED.  This will later allow us
+     to determine whether a WCP is rightfully being freed (see
+     `gw_test_cleanup_aggregating_obj ()' below).  */
+  aggregated->ref_count++;
+
+  return obj;
+}
+
+void
+gw_test_make_aggregating_obj_alt (gwTestAggregatingObj *aggregated,
+				  gwTestAggregatingObj **result)
+{
+  /* The point here is just to make sure that aggregation also works fine
+     when the aggregating object is an `out' argument instead of the return
+     value.  */
+  *result = gw_test_make_aggregating_obj (aggregated);
+}
+
+gwTestAggregatingObj *
+gw_test_get_aggregated_obj (gwTestAggregatingObj *obj)
+{
+  /* Hopefully, OBJ->AGGREGATED has not been freed, even if the Scheme code
+     no longer holds any reference to it.  Holding (on the Scheme side) a
+     SMOB to OBJ should be enough to keep both OBJ and OBJ->AGGREGATED.  */
+  return obj->aggregated;
+}
+
+
+
+/* This function automatically gets called when WCPs wrapping
+   `gwTestAggregatingObj' C objects get GC'd.  */
+size_t
+gw_test_cleanup_aggregating_obj (void *wcp)
+{
+  gwTestAggregatingObj *obj;
+
+#ifdef DEBUG
+  printf ("%s: %p\n", __FUNCTION__, wcp);
+#endif
+
+  /* Does this break C99 strict aliasing rules?  I guess no, although the
+     spec (6.5, par. 7) doesn't clearly mention this.  (XXX)  */
+  obj = (gwTestAggregatingObj *)wcp;
+
+  if (obj->aggregated)
+    {
+      /* Decrement the reference counter of the object aggregated by OBJ.  */
+#ifdef DEBUG
+      fprintf (stderr, "object %p aggregates %p\n",
+	       obj, obj->aggregated);
+#endif
+
+      if (obj->aggregated->ref_count == 0)
+	abort ();
+
+      obj->aggregated->ref_count--;
+    }
+
+  /* OBJ itself should normally not be GC'd unless all the WCPs that
+     aggregate it have already been freed (in which case its REF_COUNT is
+     zero).  */
+  if (obj->ref_count > 0)
+    {
+      fprintf (stderr, "trying to clean up aggregating object @ %p which is\n"
+	       "still referenced %u times!\n",
+	       obj, obj->ref_count);
+      abort ();
+    }
+
+  obj->aggregated = NULL;
+  obj->ref_count = 0;
+
+#ifdef DEBUG
+  printf ("freeing object @ %p\n", obj);
+#endif
+  free (obj);
+
+  return sizeof (gwTestAggregatingObj);
+}
